* This is an example of When-based validation which permits 
* vast flexibility in screen ergonomics (e.g. allowing pressing 
* of an <UNDO> button, bypassing VALID routines).

* This is an improvement over the examples that were in my FoxTalk Feb. '95 
* article about non-modal validation. It includes revalidation of all fields 
* when <SAVE> is pressed and corrects for a possible bug caused by fields that 
* do not do the When-based validation. <SAVE> revalidation is critical in case 
* the user bypassed a whole series of @..GET objects using the mouse.

* With When-based non-modal validation (WBNMV), field validation is not performed
* in the VALID of the data entry field. INSTEAD, the validation code is executed
* in the WHEN routine of the NEXT field!! This is accomplished using a object
* name token (in this example, m.sPrevGet), which is passed from object to object,
* and the native FoxPro functions VARREAD(), OBJNUM() and _CUROBJ. 

* The WBNMV method will work in FoxPro 2.x and 3.x, DOS, Windows, MacIntosh and 
* Unix, as well as any Xbase which has the equivalent of the FoxPro screen object
* functions VARREAD(), OBJNUM() and _CUROBJ.

* Note: every edit session must reset the m.sPrevGet to blank for this to properly work.

*==============begin code
SET SYSMENU AUTO
SET READBORDER ON
SET TALK OFF
SET KEYCOMP TO WINDOWS

CLEAR
MODIFY WINDOW SCREEN FONT 'Arial',9

sPrevGet = ''  && This is the token for the previous get
STORE SPACE(20) TO cVar1, cVar2, cVar3 && data entry fields
STORE '' TO sBU1, sBU2, sBU3  && and their 'revert' states
STORE 1 TO nSaveButt, nUndoButt, nEditButt, nQuitButt, nEscaButt && button vars
STORE .F. TO is_editmode  && a global flag

@ 1,1 GET cVar1 WHEN is_editmode AND vld( )
@ 3,1 GET cVar2 WHEN is_editmode AND vld( )
@ 5,1 GET cVar3 WHEN is_editmode AND vld( )
@ 7,1 GET nSaveButt PICTURE '@*N \<Save' ;
  WHEN is_editmode ;
   AND vld() ;
  VALID save() ;
  DISABLE SIZE 1.5, 6
@ 9,1 GET nUndoButt PICTURE '@*N \<Undo' ;
   WHEN is_editmode AND vld() ;
   VALID undo() DISABLE SIZE 1.5, 6
@ 11,1 GET nEditButt PICTURE '@*N \<Edit' ;
   WHEN !is_editmode VALID edit() SIZE 1.5, 6
@ 13,1 GET nQuitButt PICTURE '@* \<Quit' ;
   WHEN !is_editmode VALID QuitButt() SIZE 1.5, 6
@ 15,1 SAY "Note: you can press the <Undo> Button at any time!!!" SIZE 3, 30

READ CYCLE COLOR ,GR+/R 

RETURN

*------------------------------------
FUNCTION QuitButt
*------------------------------------
is_quitmode = .T.
CLEAR READ

RETURN
*------------------------------------
* FUNCTION vld
* the when-based validation routine 
* which contains all the _critical_ data entry rules
*------------------------------------
FUNCTION vld
was_fld_valid = .T.  && default

DO CASE
CASE VARREAD()=sPrevGet
  *--error trap, do nothing - this prevents an endless loop, must keep this!!!
CASE "UNDO" $ VARREAD()  
  * do nothing
OTHERWISE
  PRIVATE n_vld_loops, nCurLoop
  IF "NSAVEBUTT" $ VARREAD()
    n_vld_loops=2  && need 2 loops for save button
  ELSE
    n_vld_loops=1  && only one pass thru the rules for ordinary field movement
  ENDIF
  *-- if n_vld_loops = 2, then we will also do <SAVE> button revalidation of all fields!!
  FOR nCurLoop = 1 TO n_vld_loops
    
     IF UPPER(sPrevGet )="CVAR1" ;
        OR (nCurLoop=2 AND "NSAVEBUTT" $ VARREAD() AND was_fld_valid)
       *--validate 1st field
       was_fld_valid = !EMPTY(m.cVar1)
       IF !was_fld_valid
         ?? CHR(7)
         WAIT WINDOW NOWAIT "Please enter data into field 1"
         _CUROBJ=OBJNUM(m.cVar1) && redirect back to the offending field
       ENDIF
     ENDIF
     IF UPPER(sPrevGet )="CVAR2" ;
        OR (nCurLoop=2 AND "NSAVEBUTT" $ VARREAD() AND was_fld_valid)
       *--validate 2nd field
       was_fld_valid = !EMPTY(m.cVar2)
       IF !was_fld_valid
         ?? CHR(7)
         WAIT WINDOW NOWAIT "Please enter data into field 2"
         _CUROBJ=OBJNUM(m.cVar2)  && redirect back to the offending field
       ENDIF
     ENDIF
     IF UPPER(sPrevGet )="CVAR3" ;
        OR (nCurLoop=2 AND "NSAVEBUTT" $ VARREAD() AND was_fld_valid)
       *--validate 3rd field 
       was_fld_valid = !EMPTY(m.cVar3) && redirect back to the offending field
       IF !was_fld_valid
         ?? CHR(7)
         WAIT WINDOW NOWAIT "Please enter data into field 3"
         _CUROBJ=OBJNUM(m.cVar3)
       ENDIF
     ENDIF
 
  IF !was_fld_valid
     EXIT
  ENDIF
  ENDFOR

ENDCASE

IF was_fld_valid AND !( "UNDO" $ VARREAD() OR "SAVE" $ VARREAD() )
  *--we have validated the previous get ok AND we want to assign this GET TO s_prev_GET
  sPrevGet = VARREAD() && reset GET validation token
ENDIF

RETURN
*==============================
FUNCTION save
*==============================
WAIT WINDOW NOWAIT "Save the changes"
SHOW GET nSaveButt,1 DISABLE
SHOW GET nUndoButt,1 DISABLE
SHOW GET nEscaButt,1 DISABLE
SHOW GET nEditButt,1 ENABLE
SHOW GET nQuitButt,1 ENABLE
is_editmode = .F.

RETURN
*==============================
FUNCTION Undo
*==============================
WAIT WINDOW NOWAIT "Undo the changes"
STORE SPACE(20) TO cVar1, cVar2, cVar3
is_editmode = .F.
sPrevGet = '' && be sure to reset it!!

SHOW GET nSaveButt,1 DISABLE
SHOW GET nUndoButt,1 DISABLE
SHOW GET nEscaButt,1 DISABLE
SHOW GET nEditButt,1 ENABLE
SHOW GET nQuitButt,1 ENABLE
cVar1 = sBU1 
cVar2 = sBU2 
cVar3 = sBU3 
SHOW GETS

RETURN
*==============================
FUNCTION Edit
*==============================
WAIT WINDOW NOWAIT "Edit the record"
SHOW GET nSaveButt,1 ENABLE
SHOW GET nUndoButt,1 ENABLE
SHOW GET nEscaButt,1 ENABLE
SHOW GET nEditButt,1 DISABLE
SHOW GET nQuitButt,1 DISABLE
*--need these for <UNDO> Button
sBU1 = cVar1
sBU2 = cVar2
sBU3 = cVar3
is_editmode = .T.

* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*--must do this to prevent weird results!!!!
sPrevGet = '' && be sure to reset it!!

RETURN